/* globals Property, lInfo, lWarn, Utf8 */

/**
 * Stores a property parameter without any type check
 * @param path [in] property path
 * @param value [in] property value
 */
Property.setSavedProperty = function (path, value) {
    try {
        if (lInfo != null) {
            lInfo.logln('store at |' + path + '| value: |' + value + '|');
        }
        Property.setProperty(path, value);
        Property.setFlag(path, 'ARCHIVE', true);
    } catch (e) {
        if (lWarn != null) {
            lWarn.logln('Exception in Property.setSavedProperty ' + e + ' Path ' + path + ' value ' + value);
        }
    }
};

/**
 * Store a integer property parameter.
 * converts provided value whenever possible to an integer. Discard the the value if this is not possible.
 * @param path [in] property path
 * @param value [in] value to store should be integer
 * @returns {Boolean} returns false if value could not be converted to integer
 */
function checkAndStoreIntParameter (path, value) {
    try {
        if (lInfo != null) {
            lInfo.logln('store integer at |' + path + '| value: |' + value + '|');
        }
        var sucessCode = false;
        var intValue = parseInt('' + value, 10);
        if (!isNaN(intValue)) {
            Property.setProperty(path, intValue);
            Property.setFlag(path, 'ARCHIVE', true);
            sucessCode = true;
        } else {
            lWarn.logln('Typ-Fehler beim Speichern der Property ' + path);
        }
        return sucessCode;
    } catch (e) {
        if (lWarn != null) {
            lWarn.logln('Exception beim checkAndStoreIntParameter ' + e + ' Path ' + path + ' value ' + value);
        }
        return false;
    }
}

/**
 * Store a string property parameter.
 * check weather the provided value is a string. In case it's a number it is converted to string,
 * in any other cases it is discarded
 * @param path [in] property path
 * @param value [in] value to store should be string or at least a number
 * @returns {Boolean} returns false if value could not be converted
 */
function checkAndStoreStringParameter (path, value) {
    try {
        if (lInfo != null) {
            lInfo.logln('store string at |' + path + '| value: |' + value + '|');
        }
        var sucessCode = false;
        var stringValue = typeof value === 'number' ? String(value) : value;
        if (typeof stringValue === 'string') {
            Property.setProperty(path, stringValue);
            Property.setFlag(path, 'ARCHIVE', true);
            sucessCode = true;
        } else if (lWarn != null) {
            lWarn.logln('Type error when saving property ' + path);
        }
        return sucessCode;
    } catch (e) {
        if (lWarn != null) {
            lWarn.logln('Exception in checkAndStoreStringParameter ' + e + ' Path ' + path + ' value ' + value);
        }
        return false;
    }
}

/**
 * Store a dsuid as string property parameter.
 * check weather the provided value is a string. In case it's a number it is converted to string,
 * if it is undefined it tries to convert dsid to dsuid, in any other cases it is discarded
 * @param path [in] property path
 * @param dsuid [in] value to store should be string
 * @param dsid [in] dsid value to be converted in dsuid if not available
 * @returns {Boolean} returns false if value could not be converted
 */
function checkAndStoreDSUIDParameter (path, dsuid, dsid) {
    try {
        if (lInfo != null) {
            lInfo.logln('store dsuid at |' + path + '| value: |' + dsuid + '|');
        }
        var successCode = false;
        var stringValue;
        if (typeof dsuid === 'string') {
            Property.setProperty(path, dsuid);
            Property.setFlag(path, 'ARCHIVE', true);
            successCode = true;
        } else if (typeof dsid === 'string') {
            stringValue = dSID2dSUID(dsid);
            if (typeof (stringValue) === 'string') {
                Property.setProperty(path, stringValue);
                Property.setFlag(path, 'ARCHIVE', true);
                successCode = true;
            }
            if (lInfo != null) {
                lInfo.logln('store converted dsid');
            }
        } else if (lWarn != null) {
            lWarn.logln('Type error when saving property ' + path);
        }
        return successCode;
    } catch (e) {
        lWarn.logln('Exception in checkAndStoreDSUIDParameter ' + e + ' Path ' + path + ' value ' + dsuid);
        return false;
    }
}

/**
 * Store a boolean property parameter.
 * convert value if possible. In case In case conversion is not possible
 * the value is discarded.
 * @param path [in] property path
 * @param value [in] value to store should be a boolean
 * @returns {Boolean} returns false if value could not be converted
 */
function checkAndStoreBoolParameter (path, value) {
    try {
        if (lInfo != null) {
            lInfo.logln('store boolean at |' + path + '| value: |' + value + '|');
        }
        var successCode = false;
        var boolValue = false;
        if (typeof value === 'boolean') {
            Property.setProperty(path, value);
            Property.setFlag(path, 'ARCHIVE', true);
            successCode = true;
        } else if (typeof value === 'string') {
            if (value === 'true') {
                boolValue = true;
            }
            Property.setProperty(path, boolValue);
            Property.setFlag(path, 'ARCHIVE', true);
            successCode = true;
        } else if (lWarn != null) {
            lWarn.logln('Type error when saving property ' + path);
        }
        return successCode;
    } catch (e) {
        lWarn.logln('Exception in checkAndStoreBoolParameter ' + e + ' Path ' + path + ' value ' + value);
        return false;
    }
}

function dSID2dSUID (dSID) {
    try {
        // copy the first 6 bytes (12 characters to the new dsuid
        var dSUID = dSID.slice(0, 12);      // eslint-disable-line no-magic-numbers
        // insert 4 bytes of 0
        dSUID += '00000000';
        // append the remaining 6 bytes (12 chars) of the dsid
        dSUID += dSID.slice(12, 24);      // eslint-disable-line no-magic-numbers
        // finaly add 1 byte of 0
        dSUID += '00';
        return dSUID;
    } catch (e) {
        if (lWarn != null) {
            lWarn.logln('Exception in store-property-helper::dSID2dSUID ' + e);
        }
        return '';
    }
}

/* Path : complete absolute path WITHOUT trailing "/" */
function putConditionParameter (path, oParameterObject) {         // eslint-disable-line no-unused-vars
    var succeded = true;
    var conditions = oParameterObject.conditions;
    if (conditions == null) {
        return succeded;
    }
    var sSubpath = path + '/conditions/';
    if (conditions.enabled != null) {
        if (conditions.enabled) {
            Property.setSavedProperty(sSubpath + 'enabled', true);
        } else {
            Property.setSavedProperty(sSubpath + 'enabled', false);
        }
    }
    var i;
    if (conditions.weekdays) {
        var sString = '';
        for (i = 0; i < conditions.weekdays.length; i++) {
            var sWeekday = conditions.weekdays[i];
            switch (sWeekday) {
                case 'SU':
                    sString += '0,';
                    break;
                case 'MO':
                    sString += '1,';
                    break;
                case 'TU':
                    sString += '2,';
                    break;
                case 'WE':
                    sString += '3,';
                    break;
                case 'TH':
                    sString += '4,';
                    break;
                case 'FR':
                    sString += '5,';
                    break;
                case 'SA':
                    sString += '6,';
                    break;
            }
        }
        if (sString) {
            sString = sString.substr(0, sString.length - 1);
            Property.setSavedProperty(sSubpath + 'weekdays', sString);
        }
    }
    if (conditions.systemState) {
        for (i = 0; i < conditions.systemState.length; i++) {
            var systemStateName = conditions.systemState[i].name;
            var systemStateValue = conditions.systemState[i].value;
            if (systemStateValue === 'false') {
                systemStateValue = false;
            }
            if (systemStateValue === 'true') {
                systemStateValue = true;
            }
            Property.setSavedProperty(sSubpath + 'states/' + systemStateName, systemStateValue);
        }
    }
    if (conditions.addonStates) {
        for (var sAddonId in conditions.addonStates) {
            if (Object.prototype.hasOwnProperty.call(conditions.addonStates, sAddonId)) {
                for (i = 0; i < conditions.addonStates[sAddonId].length; i++) {
                    var addonStateName = conditions.addonStates[sAddonId][i].name;
                    var addonStateValue = conditions.addonStates[sAddonId][i].value;
                    if (addonStateValue === 'false' || addonStateValue === false) {
                        addonStateValue = 2;
                    } else if (addonStateValue === 'true' || addonStateValue === true) {
                        addonStateValue = 1;
                    }
                    Property.setSavedProperty(sSubpath + 'addon-states/' + sAddonId + '/' + addonStateName, addonStateValue);
                }
            }
        }
    }
    if (conditions.zoneState) {
        for (i = 0; i < conditions.zoneState.length; i++) {
            var zoneID = conditions.zoneState[i].zone;
            var groupID = conditions.zoneState[i].group;
            var sceneID = conditions.zoneState[i].scene;

            succeded = checkAndStoreIntParameter(sSubpath + 'zone-states/' + i + '/zone', zoneID) ? succeded : false;
            succeded = checkAndStoreIntParameter(sSubpath + 'zone-states/' + i + '/group', groupID) ? succeded : false;
            succeded = checkAndStoreIntParameter(sSubpath + 'zone-states/' + i + '/scene', sceneID) ? succeded : false;
        }
    }
    if (conditions.timeframe) {
        for (i = 0; i < conditions.timeframe.length; i++) {
            var from = conditions.timeframe[i].start;
            var to = conditions.timeframe[i].end;
            var sFromBase = from.timeBase;
            var sFromOffset = parseInt('' + from.offset, 10);
            var sToBase = to.timeBase;
            var sToOffset = parseInt('' + to.offset, 10);
            lInfo.logln('from ' + sFromBase + ' off ' + sFromOffset + ' to  ' + sToBase + ' off ' + sToOffset);
            if (sFromBase === 'daily') {
                if (sToBase === 'daily') {
                    Property.setSavedProperty(
                        sSubpath + 'time-start',
                        Math.floor(sFromOffset / 3600) + ':' + Math.floor(sFromOffset / 60) % 60 + ':' + Math.floor(sFromOffset % 60)   // eslint-disable-line no-magic-numbers
                    );
                    Property.setSavedProperty(
                        sSubpath + 'time-end',
                        Math.floor(sToOffset / 3600) + ':' + Math.floor(sToOffset / 60) % 60 + ':' + Math.floor(sToOffset % 60)   // eslint-disable-line no-magic-numbers
                    );
                }
            }
        }
    }
    if (conditions.date) {
        for (i = 0; i < conditions.date.length; i++) {
            Property.setSavedProperty(sSubpath + 'date/' + i + '/start', conditions.date[i].start);
            Property.setSavedProperty(sSubpath + 'date/' + i + '/end', conditions.date[i].end);
            Property.setSavedProperty(sSubpath + 'date/' + i + '/rrule', conditions.date[i].rrule);
        }
    }
    return succeded;
}

/* Path : complete absolute path WITHOUT trailing "/" */
function putTriggerParameter (path, oParameterObject) {         // eslint-disable-line no-unused-vars
    var succeded = true;
    var i;
    for (i = 0; i < oParameterObject.triggers.length; i++) {
        var sSubpath = path + '/triggers/' + (i + 1) + '/';
        var trigger = oParameterObject.triggers[i];
        switch (trigger.type) {
            case 'bus-zone-scene':
            case 'zone-scene':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'zone', trigger.zone) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'group', trigger.group) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'scene', trigger.scene) ? succeded : false;
                if (trigger.forced != null) {
                    succeded = checkAndStoreBoolParameter(sSubpath + 'forced', trigger.forced) ? succeded : false;
                }
                if (trigger.dsuid != null) {
                    Property.setSavedProperty(sSubpath + 'dsuid', trigger.dsuid);
                } else {
                    Property.setSavedProperty(sSubpath + 'dsuid', -1);
                }
                break;
            case 'bus-undo-zone-scene':
            case 'undo-zone-scene':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'zone', trigger.zone) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'group', trigger.group) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'scene', trigger.scene) ? succeded : false;
                if (trigger.dsuid != null) {
                    Property.setSavedProperty(sSubpath + 'dsuid', trigger.dsuid);
                } else {
                    Property.setSavedProperty(sSubpath + 'dsuid', -1);
                }
                break;
            case 'device-msg':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreDSUIDParameter(sSubpath + 'dsuid', trigger.dsuid, trigger.dsid) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'msg', trigger.msg) ? succeded : false;
                if (trigger.buttonIndex != null) {
                    succeded = checkAndStoreIntParameter(sSubpath + 'buttonIndex', trigger.buttonIndex) ? succeded : false;
                }
                break;
            case 'device-action':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreDSUIDParameter(sSubpath + 'dsuid', trigger.dsuid, trigger.dsid) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'action', trigger.action) ? succeded : false;
                break;
            case 'device-binary-input':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'index', trigger.index) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'state', trigger.state) ? succeded : false;
                break;
            case 'device-scene':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreDSUIDParameter(sSubpath + 'dsuid', trigger.dsuid, trigger.dsid) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'scene', trigger.scene) ? succeded : false;
                break;
            case 'custom-event':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'event', trigger.event) ? succeded : false;
                break;
            case 'device-sensor':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreDSUIDParameter(sSubpath + 'dsuid', trigger.dsuid, trigger.dsid) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'eventid', trigger.eventid) ? succeded : false;
                break;
            case 'device-named-event':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreDSUIDParameter(sSubpath + 'dsuid', trigger.dsuid, trigger.dsid) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'id', trigger.id) ? succeded : false;
                break;
            case 'device-named-action':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreDSUIDParameter(sSubpath + 'dsuid', trigger.dsuid, trigger.dsid) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'id', trigger.id) ? succeded : false;
                break;
            case 'state-change':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'name', trigger.name) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'state', trigger.state) ? succeded : false;
                break;
            case 'addon-state-change':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'addon-id', trigger.addonId) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'name', trigger.name) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'state', trigger.state) ? succeded : false;
                break;
            case 'event':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', trigger.type) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'name', trigger.name) ? succeded : false;
                sSubpath += 'parameter/';
                for (var key in trigger.parameter) {
                    if (Object.prototype.hasOwnProperty.call(trigger.parameter, key)) {
                        succeded = checkAndStoreStringParameter(sSubpath + key, trigger.parameter[key]) ? succeded : false;
                    }
                }
                break;
        }
    }
    return succeded;
}

function putActionParameter (path, oParameterObject, category) {         // eslint-disable-line no-unused-vars
    var i;
    var succeded = true;
    for (i = 0; i < oParameterObject.actions.length; i++) {
        var sSubpath = path + '/actions/' + i + '/';
        var action = oParameterObject.actions[i];
        switch (action.type) {
            case 'device-moc-value':
                succeded = false;
                var j = 0;
                while (action['channelid-' + j] != null) {
                    succeded = true;
                    succeded = checkAndStoreIntParameter(sSubpath + 'channelid-' + j, action['channelid-' + j]) ? succeded : false;
                    succeded = checkAndStoreIntParameter(sSubpath + 'channelvalue-' + j, action['channelvalue-' + j]) ? succeded : false;
                    j++;
                }
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                succeded = checkAndStoreDSUIDParameter(sSubpath + 'dsuid', action.dsuid, action.dsid) ? succeded : false;
                break;
            case 'device-action':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                succeded = checkAndStoreDSUIDParameter(sSubpath + 'dsuid', action.dsuid, action.dsid) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'id', action.id) ? succeded : false;
                break;
            case 'heating-mode':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'zone', action.zone) ? succeded : false;
                if (action.reset != null) { succeded = checkAndStoreBoolParameter(sSubpath + 'reset', action.reset) ? succeded : false; }
                succeded = checkAndStoreIntParameter(sSubpath + 'mode', action.mode) ? succeded : false;
                break;
            case 'zone-scene':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'zone', action.zone) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'group', action.group) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'scene', action.scene) ? succeded : false;
                if (action.force != null) { succeded = checkAndStoreBoolParameter(sSubpath + 'force', action.force) ? succeded : false; }
                break;
            case 'undo-zone-scene':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'zone', action.zone) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'group', action.group) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'scene', action.scene) ? succeded : false;
                if (action.force != null) { succeded = checkAndStoreBoolParameter(sSubpath + 'force', action.force) ? succeded : false; }
                break;
            case 'device-scene':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                succeded = checkAndStoreDSUIDParameter(sSubpath + 'dsuid', action.dsuid, action.dsid) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'scene', action.scene) ? succeded : false;
                if (action.force != null) { succeded = checkAndStoreBoolParameter(sSubpath + 'force', action.force) ? succeded : false; }
                break;
            case 'device-value':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'value', action.value) ? succeded : false;
                succeded = checkAndStoreDSUIDParameter(sSubpath + 'dsuid', action.dsuid, action.dsid) ? succeded : false;
                break;
            case 'device-blink':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                succeded = checkAndStoreDSUIDParameter(sSubpath + 'dsuid', action.dsuid, action.dsid) ? succeded : false;
                break;
            case 'zone-blink':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'zone', action.zone) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'group', action.group) ? succeded : false;
                break;
            case 'custom-event':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'event', action.event) ? succeded : false;
                break;
            case 'url':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                action.url = action.url.replace(/&amp;/g, '&');
                succeded = checkAndStoreStringParameter(sSubpath + 'url', Utf8.encode(unescape(action.url))) ? succeded : false;
                break;
            case 'change-state':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                if (action.name != null) {
                    succeded = checkAndStoreStringParameter(sSubpath + 'statename', action.name) ? succeded : false;
                } else {
                    succeded = checkAndStoreStringParameter(sSubpath + 'statename', action.statename) ? succeded : false;
                }

                succeded = checkAndStoreStringParameter(sSubpath + 'state', action.state);
                break;
            case 'change-addon-state':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                if (action.name != null) {
                    succeded = checkAndStoreStringParameter(sSubpath + 'statename', action.name) ? succeded : false;
                } else {
                    succeded = checkAndStoreStringParameter(sSubpath + 'statename', action.statename) ? succeded : false;
                }

                succeded = checkAndStoreStringParameter(sSubpath + 'addon-id', action.addonId) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'state', action.state) ? succeded : false;
                break;
            case 'play-audio-activity':
                succeded = checkAndStoreStringParameter(sSubpath + 'type', action.type) ? succeded : false;
                succeded = checkAndStoreDSUIDParameter(sSubpath + 'dsuid', action.dsuid, action.dsid) ? succeded : false;
                succeded = checkAndStoreIntParameter(sSubpath + 'powerState', action.powerState) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'contentSource', action.contentSource) ? succeded : false;
                if (action.volume != null) {
                    succeded = checkAndStoreIntParameter(sSubpath + 'volume', action.volume) ? succeded : false;
                }
                succeded = checkAndStoreIntParameter(sSubpath + 'repeatMode', action.repeatMode || 0) ? succeded : false;
                succeded = checkAndStoreBoolParameter(sSubpath + 'shuffleMode', Boolean(action.shuffleMode)) ? succeded : false;
                succeded = checkAndStoreStringParameter(sSubpath + 'zones', action.zones) ? succeded : false;
                break;
            default:
                // unexpected type
                succeded = false;
                break;
        }
        if (action.delay != null) {
            succeded = checkAndStoreIntParameter(sSubpath + 'delay', action.delay) ? succeded : false;
        }
        if (category == null) {
            Property.setSavedProperty(sSubpath + 'category', 'manual');
        } else {
            Property.setSavedProperty(sSubpath + 'category', category);
        }
    }
    return succeded;
}
